# novacut: the collaborative video editor
# Copyright (C) 2012 Novacut Inc
#
# This file is part of `novacut`.
#
# `novacut` is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# `novacut` is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
# more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with `novacut`.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#   Jason Gerard DeRose <jderose@novacut.com>

"""
Unit tests for the `novacut.migration` module.
"""

import json
from fractions import Fraction
from unittest import TestCase

from usercouch.misc import CouchTestCase
from microfiber import Database

from novacut.timefuncs import frame_to_sample
from novacut import schema, migration


docs_s = """
[
    {
        "_id": "WPKFHBKX2LTHAIAILWAK7PEG",
        "doodle": [
            {
                "id": "shortcuts",
                "x": null,
                "y": null
            },
            {
                "id": "ZZOAZZCEJIZV36SDRNGLNP3J",
                "x": 1302,
                "y": 408
            },
            {
                "id": "N4ZQGJFX24SQH6OY6JDHXCT2",
                "x": 195,
                "y": 230
            }
        ],
        "node": {
            "src": [
                "JDQPJVFJOEPYR64FRTEULTBB",
                "GMWUVRRSBZFSD7PD6IKSQXLU",
                "7ENE3E3H2XWAETPU4E2SGMQY",
                "ZCVOKON745KTKOZTI5JKRMNP",
                "H3BN43ZC4OUI42R7K4BSQ67Q",
                "ZITFANVLNT36I4PQ54B7XTGI",
                "B56KMP2HVW4GHAXKFDYQQ5UD",
                "7BR63KHALAAFZNZJPWMUJ77X",
                "HYHA764RM4BDVBYPA7SQG4LW",
                "6VTAXG6PHUVR7NBLIBT63JCN",
                "VJF2TXN4IDEL4FHYWBTOMYPB"
            ],
            "type": "sequence"
        },
        "selected": "N4ZQGJFX24SQH6OY6JDHXCT2",
        "session_id": "1345419541-SGIO4XPFNZ4KGXK4",
        "time": 1342803135.184,
        "type": "novacut/node",
        "ver": 0
    },
    {
        "_id": "JDQPJVFJOEPYR64FRTEULTBB",
        "node": {
            "src": "F6KHESGKSVXCRXH7FEJHLJRP44QRORF7GEMIJM4YPBDSMAMF",
            "start": {
                "frame": 127
            },
            "stop": {
                "frame": 205
            },
            "stream": "both",
            "type": "slice"
        },
        "session_id": "1345340199-RXJX3JGYQP4TFLNX",
        "time": 1343379567.943,
        "type": "novacut/node",
        "ver": 0
    },
    {
        "_id": "GMWUVRRSBZFSD7PD6IKSQXLU",
        "node": {
            "src": "NYJOQ4KHVZ3SM5SODHWFAUZ5GIJXNPDLHKASMAO2K6Y4V2LT",
            "start": {
                "frame": 238
            },
            "stop": {
                "frame": 557
            },
            "stream": "both",
            "type": "slice"
        },
        "session_id": "1345201341-QNG7X3IZHXX2UH5K",
        "time": 1343379504.205,
        "type": "novacut/node",
        "ver": 0
    },
    {
        "_id": "NYJOQ4KHVZ3SM5SODHWFAUZ5GIJXNPDLHKASMAO2K6Y4V2LT",
        "bytes": 196010853,
        "channels": 2,
        "content_type": "video/quicktime",
        "ctime": 1343161881.37,
        "duration": {
            "frames": 848,
            "nanoseconds": 33920000000,
            "samples": 1628160,
            "seconds": 33.92
        },
        "ext": "mov",
        "framerate": {
            "denom": 1,
            "num": 25
        },
        "height": 1088,
        "import": {
            "batch_id": "G43C4TNHZTQID2GYXQ7AJWKY",
            "import_id": "AQ3UCVFNDX3MD7KD7HPTBSGC",
            "machine_id": "PIZ6WYOE72XOHEXMHLMXIMEW",
            "mtime": 1343183514,
            "project_id": "PQUS2SCGWLKXE23WXF2CXVRJ",
            "src": "/run/media/jderose/EOS_DIGITAL/DCIM/100EOS5D/MVI_9316.MOV"
        },
        "media": "video",
        "meta": {
            "aperture": 3.5,
            "camera": "Canon EOS 5D Mark II",
            "camera_serial": "0820500998",
            "canon_thm": "HUDPYMFK4YMB2LDKGMBEKLAENFMNMRUSBLBINOUEGH6XCUPQ",
            "focal_length": "50.0 mm",
            "iso": 1600,
            "lens": "Canon EF 50mm f/1.2L",
            "shutter": "1/60"
        },
        "name": "MVI_9316.MOV",
        "origin": "user",
        "samplerate": 48000,
        "session_id": "1343379450-N2F6ZXTLZM46ASCJ",
        "tags": {},
        "time": 1343191600.457724,
        "type": "dmedia/file",
        "ver": 0,
        "width": 1920
    },
    {
        "_id": "F6KHESGKSVXCRXH7FEJHLJRP44QRORF7GEMIJM4YPBDSMAMF",
        "bytes": 122207749,
        "channels": 2,
        "content_type": "video/quicktime",
        "ctime": 1343163072.67,
        "duration": {
            "frames": 536,
            "nanoseconds": 21440000000,
            "samples": 1029120,
            "seconds": 21.44
        },
        "ext": "mov",
        "framerate": {
            "denom": 1,
            "num": 25
        },
        "height": 1088,
        "import": {
            "batch_id": "G43C4TNHZTQID2GYXQ7AJWKY",
            "import_id": "AQ3UCVFNDX3MD7KD7HPTBSGC",
            "machine_id": "PIZ6WYOE72XOHEXMHLMXIMEW",
            "mtime": 1343184694,
            "project_id": "PQUS2SCGWLKXE23WXF2CXVRJ",
            "src": "/run/media/jderose/EOS_DIGITAL/DCIM/100EOS5D/MVI_9320.MOV"
        },
        "media": "video",
        "meta": {
            "aperture": 3.5,
            "camera": "Canon EOS 5D Mark II",
            "camera_serial": "0820500998",
            "canon_thm": "7NB57GH3GDSEPIGI7RTRFTZ2QYU2GZMTMECUEMFO4XLRWEVH",
            "focal_length": "100.0 mm",
            "iso": 1600,
            "lens": "Canon EF 100mm f/2.8L Macro IS USM",
            "shutter": "1/60"
        },
        "name": "MVI_9320.MOV",
        "origin": "user",
        "samplerate": 48000,
        "session_id": "1343379450-N2F6ZXTLZM46ASCJ",
        "tags": {},
        "time": 1343191736.768652,
        "type": "dmedia/file",
        "ver": 0,
        "width": 1920
    }
]
"""


def get_id_map():
    return {
        'NYJOQ4KHVZ3SM5SODHWFAUZ5GIJXNPDLHKASMAO2K6Y4V2LT': 'SLNSQE6YLRLBVWFCLNBBFSAKVWXM4GB3S6IMGKVPD4EI6UJR',
        'F6KHESGKSVXCRXH7FEJHLJRP44QRORF7GEMIJM4YPBDSMAMF': 'C5NJAVGPS9H8LXRYMYVGB8J6IHIIF8CTTWYOT5G75ELXAEYA',
    }


class TestFunctions(TestCase):
    def test_migrate_project(self):
        old = {
            "_id": "AYDDLRKHDL7YJ5TR6JLCMO3D",
            "_rev": "11-25500cd9e63b08cd79c220b66982f7d3",
            "atime": 1366294024.9010231,
            "db_name": "novacut-0-ayddlrkhdl7yj5tr6jlcmo3d",
            "isdeleted": True,
            "time": 1354124043.327159,
            "title": "Test",
            "type": "novacut/project",
            "ver": 0,
        }
        new = migration.migrate_project(old)
        self.assertIsNot(old, new)
        self.assertEqual(new, {
            "_id": "3R66EKDA6EYRCWMKXCE5FHU6",
            "atime": 1366294024.9010231,
            "db_name": "novacut-1-3r66ekda6eyrcwmkxce5fhu6",
            "isdeleted": True,
            "time": 1354124043.327159,
            "title": "Test",
            "type": "novacut/project",
        })

        # With root_id:
        old = {
            "_id": "AYDDLRKHDL7YJ5TR6JLCMO3D",
            "_rev": "11-25500cd9e63b08cd79c220b66982f7d3",
            "atime": 1366294024.9010231,
            "db_name": "novacut-0-ayddlrkhdl7yj5tr6jlcmo3d",
            "isdeleted": True,
            "time": 1354124043.327159,
            "title": "Test",
            "type": "novacut/project",
            "ver": 0,
            "root_id": "ZG72S2W24LVOVHQIPGECZSYT"
        }
        new = migration.migrate_project(old)
        self.assertIsNot(old, new)
        self.assertEqual(new, {
            "_id": "3R66EKDA6EYRCWMKXCE5FHU6",
            "atime": 1366294024.9010231,
            "db_name": "novacut-1-3r66ekda6eyrcwmkxce5fhu6",
            "isdeleted": True,
            "time": 1354124043.327159,
            "title": "Test",
            "type": "novacut/project",
            "root_id": "S9YTLTPTVEOHOAJBI975SLRM"
        })

    def test_migrate_sequence(self):
        old = json.loads(docs_s)[0]
        new = migration.migrate_sequence(old)
        self.assertIsNot(old, new)
        self.assertEqual(old, json.loads(docs_s)[0])
        self.assertEqual(new, {
            '_id': 'PID8A4DQTEMA3B3BEP3DYI79',
            'type': 'novacut/node',
            'time': 1342803135.184,
            'selected': 'GVSJ9C8QTVLJAXHRXC6AQ5MT',
            'node': {
                'type': 'video/sequence',
                'src': [
                    'C6JICO8CH7IRKXV8KM7NEM44',
                    '9FPNOKKL4S8L6YI6XBDLJQEN',
                    'Y7G7U7UATQP37MINV7TL9FJR',
                    'S5OHDHGYVWDMDHSMBWCDKFGI',
                    'AU4GVUS5VHNBVTKYDV4LJXYJ',
                    'SBM83GOEGMUXBVIJWV4YQM9B',
                    '4WXDFITAOPV9A3QD86RJJWN6',
                    'Y4KXUDA3E338SGSCIPFNCYYQ',
                    'ARA3YXVKFV46O4RI3YLJ9VEP',
                    'XOM3Q9XIANOKYG4EB4MXUC5G',
                    'OC8TMQGVB67EV8ARP4MHFRI4',
                ],
            },
            'audio': [],
            'doodle': [
                {'id': 'SSH3SS57CBSOUXL6KG9EGIUC', 'x': 1302, 'y': 408},
                {'id': 'GVSJ9C8QTVLJAXHRXC6AQ5MT', 'x':  195, 'y': 230},
            ], 
        })
        schema.check_video_sequence(new)

    def test_migrate_slice(self):
        old = json.loads(docs_s)[1]
        id_map = get_id_map()
        new = migration.migrate_slice(old, id_map)
        self.assertIsNot(old, new)
        self.assertEqual(old, json.loads(docs_s)[1])
        self.assertEqual(new, {
            '_id': 'C6JICO8CH7IRKXV8KM7NEM44',
            'type': 'novacut/node',
            'time': 1343379567.943,
            'audio': [],
            'node': {
                'type': 'video/slice',
                'src': 'C5NJAVGPS9H8LXRYMYVGB8J6IHIIF8CTTWYOT5G75ELXAEYA',
                'start': 127,
                'stop': 205,
            },
        })
        schema.check_video_slice(new)

        old = json.loads(docs_s)[2]
        id_map = get_id_map()
        new = migration.migrate_slice(old, id_map)
        self.assertIsNot(old, new)
        self.assertEqual(old, json.loads(docs_s)[2])
        self.assertEqual(new, {
            '_id': '9FPNOKKL4S8L6YI6XBDLJQEN',
            'type': 'novacut/node',
            'time': 1343379504.205,
            'audio': [],
            'node': {
                'type': 'video/slice',
                'src': 'SLNSQE6YLRLBVWFCLNBBFSAKVWXM4GB3S6IMGKVPD4EI6UJR',
                'start': 238,
                'stop': 557,
            },
        })
        schema.check_video_slice(new)

    def test_migrate_dmedia_file(self):
        id_map = get_id_map()

        old = json.loads(docs_s)[3]
        new = migration.migrate_dmedia_file(old, get_id_map())
        self.assertIsNot(old, new)
        self.assertEqual(old, json.loads(docs_s)[3])
        self.assertEqual(new['_id'],
            'SLNSQE6YLRLBVWFCLNBBFSAKVWXM4GB3S6IMGKVPD4EI6UJR'
        )
        self.assertNotIn('_rev', new)

        old = json.loads(docs_s)[4]
        new = migration.migrate_dmedia_file(old, get_id_map())
        self.assertIsNot(old, new)
        self.assertEqual(old, json.loads(docs_s)[4])
        self.assertEqual(new['_id'],
            'C5NJAVGPS9H8LXRYMYVGB8J6IHIIF8CTTWYOT5G75ELXAEYA',
        )
        self.assertNotIn('_rev', new)
